/*  An "Expressive" compiler
 *
 *  Manages  information about the position within a source file
 *  Matthias Zenger, Michel Schinz
 *
 *  $Id: Position.java,v 1.2 2003/03/20 09:44:11 mihaylov Exp $
 */

package expressive;

/**
 * This class contains the methods to encode the position within 
 * a source file (line & column) in one 'int' value.
 */
public final class Position {

    /** Number of bits reserved for encoding the column. */
    private static final int columnBits = 12;

    /** Bit mask for encoding and decoding the column. */
    private static final int columnMask = (1 << columnBits) - 1;

    /** The undefined position. */
    public static final int UNDEFINED = 0;

    /** Position of the first character of all files. */
    public static final int FIRST = (1 << columnBits) | 1;

    /** Encode the line and coolumn in one 'int'. */
    public static int encode(int line, int col) {
        return (line << columnBits) | (col & columnMask);
    }

    /** Extract the line number from an encoded position. */
    public static int line(int pos) {
        return pos >>> columnBits;
    }

    /** Extract the column number from an encoded position. */
    public static int column(int pos) {
        return pos & columnMask;
    }

}
